/*
 * DC Motor Control with Arduino and MotorShield
 * Using one PC6-C-4 decoder circuit board (LS7184 chip) and one interrupt pin
 % Need 12 V external power supply
 * 
 * October 4, 2009 (v. 1.3)
 */

#include <AFMotor.h>                // motorshield library

AF_DCMotor motor(2, MOTOR12_64KHZ); // create motor #2, 64KHz pwm
// Double check that the motor number matches your motor shield's setup

#define ClockPin 2      // clock interrupt pin
#define UpDownPin 13    // direction pin
int encoder0Pos = 0;  // encoder position
int count = 0;
int updown = 0;
int incomingByte;
unsigned long time_1, time_2, dt;

float vel, oldVel, error;
long oldPos;
long setPoint = 0;
int command = 0;
long input = 0;

float Kp, Ki, Kd; // controller gains

void setup() {
  Serial.begin(9600);            // set up Serial library at 9600 bps
  Kp = 0.0;
  Ki = 0.0;
  Kd = 0.0;
  
  pinMode(ClockPin, INPUT);
  pinMode(UpDownPin, INPUT); 

// encoder pin on interrupt 0 (pin 2)
  attachInterrupt(0, doEncoder, CHANGE);
  time_1 = millis();              // read the initial time stamp
}

void loop() {
  // Serial.println("Motor Control");
  time_2 = millis();              // read the current time stamp
  dt = time_2 - time_1;           // compute delta time
  time_1 = time_2;                // reassign new time_1
  
  //*** Remeber time is in milliseconds!!!
  
  // update state variables for use in PID controller
  vel = (float) (encoder0Pos - oldPos) / dt;  // velocity estimate in ms
  error = setPoint - encoder0Pos;             // position error in counts
  
  // reassign state variables
  oldPos = encoder0Pos;
  
  //***********************************
  // Insert controller here
  // command = ????
  //
  // remember command should be an integer
  //***********************************
  
  // limit outputs to maximum allowable in setSpeed function
  if (command > 255)
  {
    command = 255;
  }
  else if (command < -255)
  {
    command = -255;
  }
  
  // setspeed only takes positive values and a direction.
  // this takes care of the bookkeeping
  if (command > 0)
  {
    motor.setSpeed(command);
    motor.run(FORWARD);
  }
  else
  {
    motor.setSpeed(-command);
    motor.run(BACKWARD);
  }
  
  Serial.print(time_2, DEC);           // print out time since the beginning in ms
  Serial.print(" ");                   // space separator
  Serial.println(encoder0Pos, DEC);    // print out the encoder position
  // Adjust here if you want to print out additional variables.
  // Remeber what you printed when you go to matlab!


// Serial input to adjust the setpoint
if (Serial.available() > 0)
{
  while (Serial.available() > 0) {    
	// read the incoming byte:
	incomingByte = Serial.read();

        switch (incomingByte)
        {
        case '1':
          input *= 10;
          input += 1;
          break;
        case '2':
          input *= 10;
          input += 2;
          break;
        case '3':
          input *= 10;
          input += 3;
          break;
        case '4':
          input *= 10;
          input += 4;
          break;
        case '5':
          input *= 10;
          input += 5;
          break;
        case '6':
          input *= 10;
          input += 6;
          break;
        case '7':
          input *= 10;
          input += 7;
          break;
        case '8':
          input *= 10;
          input += 8;
          break;
        case '9':
          input *= 10;
          input += 9;
          break;
        case '0':
          input *= 10;
          break; 
        case ';':
          setPoint = input;
          input = 0;
	}
  }
}

 // Function generator section will override manual input
  // choose the sine wave, square wave, or neither.
  
  // Square Wave
  /*
  if (time_2 % 10000 < 5000)
  {
    setPoint = 8000;
  }
  else
  {
    setPoint = 10000;
  }
  */
  
  // Sine Wave
  //setPoint = 1000*sin(.2*(2*3.14159)/1000*(time_2)) + 8000;
  
}

// Interrupt routine
void doEncoder(){
  count = digitalRead(ClockPin);       // read x4 clock output
  updown = digitalRead(UpDownPin);     // read the direction of motor
  if (count == LOW) {                  // a pulse is detected 
    if (updown == HIGH) {              // cw direction
      encoder0Pos = encoder0Pos + 1;   // increment motor position
    }
    else {                              // ccw direction
      encoder0Pos = encoder0Pos - 1;    // decrement motor position
    }
  }
}


